<?php

namespace Teknisa\DF\Controllers;

use NFePHP\NFe\MakeNFe;
use NFePHP\NFe\ToolsNFe;
use Teknisa\DF\Services\TransmissaoNfeService;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;

class TransmissaoNfeController extends \Teknisa\DF\Controllers\TeknisaCrud
{
    protected $transmissaoNfeService;

    public function __construct(TransmissaoNfeService $transmissaoNfeService)
    {
        $this->transmissaoNfeService = $transmissaoNfeService;
    }

    public function montaConfigTransmissao (Request\Filter $request, Response $response) {
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $row = $requestConditions[0]["value"];
        
        try {
            if(isset($row["MANIFESTACAO"])) $manifestacao = true;
            else $manifestacao = false;

            $content = $this->transmissaoNfeService->montaConfigTransmissao($row['CDEMPRESA'], $row['CDINSCESTA'], $row['CDFILIAL']);
            
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('config_transmissao', array('config' => $content)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function statusServico (Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $notasfiscais = $requestConditions[0]["value"];
        $config = $requestConditions[1]["value"];

        $statusServico = $this->transmissaoNfeService->statusServico($notasfiscais, $config);

        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('statusServico', $statusServico));
    }

    public function transmiteNotas (Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $notasfiscais = $requestConditions[0]["value"];
        $config = $requestConditions[1]["value"];
        $tipo = $requestConditions[2]["value"];

        $transmissao = $this->transmissaoNfeService->transmitirNotas($notasfiscais, $config, $tipo);

        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('transmiteNotas', $transmissao));
    }

    public function atualizaLote (Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $lotesMarcados = $requestConditions[0]["value"];
        $config = $requestConditions[1]["value"];

        $atualizaLote = $this->transmissaoNfeService->atualizaLote($lotesMarcados, $config, $response);

        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('atualizaLote', $atualizaLote));
    }

    public function cancelarTransmissao (Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $row = $requestConditions[0]["value"];
        $config = $requestConditions[1]["value"];

        $cancelar = $this->transmissaoNfeService->cancelarTransmissao($row, $config);

        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('cancelarTransmissao', $cancelar));
    }

    public function enviarCartaCorrecao (Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $notafiscal = $requestConditions[0]["value"];
        $dscorrecaonfe = $requestConditions[1]["value"];
        $nrseqcce = $requestConditions[2]["value"];
        $config = $requestConditions[3]["value"];

        try {
            $cartaCorrecao = $this->transmissaoNfeService->enviarCartaCorrecao($notafiscal, $dscorrecaonfe, $nrseqcce, $config);

            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('enviarCartaCorrecao', $cartaCorrecao));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function visualizarDanfe (Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $notasMarcadas = $requestConditions[0]["value"];
        $config = $requestConditions[1]["value"];

        try {
            $visualizarDanfe = $this->transmissaoNfeService->visualizarDanfe($notasMarcadas, $config);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('visualizarDanfe', $visualizarDanfe));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));

        }
    }

    public function downloadDanfe (Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $notasMarcadas = $requestConditions[0]["value"];
        $config = $requestConditions[1]["value"];

        try {
            $downloadDanfe = $this->transmissaoNfeService->downloadDanfe($notasMarcadas, $config);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('downloadDanfe', array($downloadDanfe)));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function downloadXML (Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $notasMarcadas = $requestConditions[0]["value"];
        $config = $requestConditions[1]["value"];

        try {
            $downloadXML = $this->transmissaoNfeService->downloadXML($notasMarcadas, $config);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('downloadXML', array($downloadXML)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function enviarEmailNF (Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $nota = $requestConditions[0]["value"];
        $config = $requestConditions[1]["value"];
        $tipo = $requestConditions[2]["value"];

        $enviarEmailNF = $this->transmissaoNfeService->enviarEmailNF($nota, $config, $tipo);

        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('enviarEmailNF', $enviarEmailNF));
    }
    
    public function enviarEmailCC(Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $nota = $requestConditions[0]["value"];
        $tipo = $requestConditions[1]["value"];
        $nrseqcce = $requestConditions[2]["value"];
        $config = $requestConditions[3]["value"];


        $enviarEmailCC = $this->transmissaoNfeService->enviarEmailCC($nota, $tipo, $nrseqcce, $config);

        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('enviarEmailCC', $enviarEmailCC));
    }

    public function maxNrseqcce(Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $filial = $requestConditions[0]["value"];
        $lancamento = $requestConditions[1]["value"];

        $maxNrseqcce = $this->transmissaoNfeService->getMaxNrseqcce($filial, $lancamento);

        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('maxNrseqcce', array($maxNrseqcce)));
    }
    
    public function verificaTipoOperFili(Request\Row $request, Response $response){
        $row = $request->getRow();
        $liberado = $this->transmissaoNfeService->verificaTipoOperFili($row["CDTIPOOPER"], $row["CDFILIAL"], $row["IDENTRSAIDOP"]);
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('liberado', array($liberado)));
    }
    
    public function consultaSituacaoNF(Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $nota = $requestConditions[0]["value"];
        $config = $requestConditions[1]["value"];
        $this->transmissaoNfeService->consultaSituacaoNF($nota, $config, $response);
    }
    
    public function gerarXmlNfe(Request\DataSet $request, Response $response){
        $notas = $request->getDataSet()->getRows();
        $zipName = $this->transmissaoNfeService->gerarXmlNfe($notas);
        $response->addMessage(new Response\Message($zipName));
    }
}