<?php

namespace Teknisa\DF\Modules\Saida\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\DF\Modules\Saida\Services\ManifestoDocumentoEletronicoService;
    
class ManifestoDocumentoEletronicoController extends \Teknisa\DF\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $parameterBag;
    protected $dataSourceName = "mdfe";
    protected $novoCodigo;
    protected $manifestoDocumentoEletronicoService;
    const TIPO_ARQ_XML_DADOS = 'D';
    const TIPO_ARQ_XML_AUTORIZACAO = 'A';
    const TIPO_ARQ_XML_CANCELAMENTO = 'C';
    const TIPO_ARQ_XML_CARTA_CORRECAO = 'E';
    
    const NOVO_CODIGO_LANC_MDFE = "LANCTOMDFE";
    
    public function __construct($crudService, $parameterBag, ManifestoDocumentoEletronicoService $manifestoDocumentoEletronicoService, $novoCodigo, $environment) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->manifestoDocumentoEletronicoService = $manifestoDocumentoEletronicoService;
        $this->novoCodigo = $novoCodigo;
        $this->environment = $environment;
    }
    
    public function getDefaultMdfe(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $cdEmpresa = isset($row["CDEMPRESA"]) ? $row["CDEMPRESA"] : ''; 
        
        $mdfe = $this->manifestoDocumentoEletronicoService->getDefaultMdfe($cdEmpresa);
        
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('mdfe', $mdfe));
    
    }
    
    public function salvaSeguroCarga(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        if($row["__is_new"]) {
            $this->manifestoDocumentoEletronicoService->setSequencialSeguroCarga($row);
            $requestComSequencial = new Request\Row($row, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        }
        else {
            $requestComSequencial = $request;
        }
        
        $this->dataSourceName = "segurocargamdfe";
        
        parent::save($requestComSequencial, $response);
        
    }
    
    public function salvarAverbacao(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        if($row["__is_new"]) {
            $this->manifestoDocumentoEletronicoService->setSequencialAverbacao($row);
            $requestComSequencial = new Request\Row($row, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        }
        else {
            $requestComSequencial = $request;
        }
        
        $this->dataSourceName = "averbacaomdfe";
        
        parent::save($requestComSequencial, $response);
        
    }

    public function salvarDadosAdicMdfe(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        if($row["__is_new"]) {
            $this->manifestoDocumentoEletronicoService->setSequencialDadosAdic($row);
            $requestComSequencial = new Request\Row($row, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        }
        else {
            $requestComSequencial = $request;
        }
        
        $this->dataSourceName = "dadosadicmdfe";
        
        parent::save($requestComSequencial, $response);
        
    }

    public function salvarMDFE(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        if($row["__is_new"] === true) {
            $row["NRLANCTOMDFE"] = $this->novoCodigo->getCodeByTableName(self::NOVO_CODIGO_LANC_MDFE.$row["CDVIATURA"], $this->environment->getNrOrgTrab());
        }
        $row["NRLOTEMDFE"] = $row["CDEMPRESA"];
        $this->dataSourceName = "mdfe";
        parent::save($request, $response);
        if($row["__is_new"] === true) {
            $this->manifestoDocumentoEletronicoService->insereCarregamentoMdfe($row);
        }
        $response->addMessage(new Response\Message($row));
    }
    
    public function excluirMDFE(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        $this->manifestoDocumentoEletronicoService->limparDadosMdfe($row);
        $this->dataSourceName = "mdfe";
        parent::delete($request, $response);
    }
    
    public function saveNotaMdfe(Request\Dataset $request, Response $response) {
        $dataset = $request->getDataset();
        $rows = $dataset->getRows();
        
        if ($rows) {
            $rowSequencial = $rows[0];
            $this->manifestoDocumentoEletronicoService->setSequencialNotasRelac($rowSequencial);
            $nrseqrelacnf = $rowSequencial["NRSEQRELACNF"];
            $nrordementrega = $rowSequencial["NRORDEMENTREGA"];
        }
        
        foreach($rows as &$row) {
            $row["NRSEQRELACNF"] = str_pad($nrseqrelacnf++, 4, "0", STR_PAD_LEFT);
            $row["NRORDEMENTREGA"] = str_pad($nrordementrega++, 3, "0", STR_PAD_LEFT);
        }
        
        $this->dataSourceName = "nfrelacmdfe";
        parent::save($request, $response);
        $this->manifestoDocumentoEletronicoService->atualizaDadosNotas($row);
        $this->atualizaPercurso($request, $row);
    }
    
    public function mudarOrdemNotaMdfe (Request\Row $request, Response $response) {
        $row = $request->getRow();
        $this->manifestoDocumentoEletronicoService->mudarOrdemNota($row);
    }
    
    public function excluirNotaMdfe (Request\Row $request, Response $response) {
        $row = $request->getRow();
        $this->manifestoDocumentoEletronicoService->excluirNota($row);
        $this->atualizaPercurso($request, $row);
    }
    
    public function excluirNotaMdfeMultiplo (Request\Dataset $request, Response $response) {
        $dataset = $request->getDataset();
        $rows = $dataset->getRows();
        foreach($rows as $row) {
            $this->manifestoDocumentoEletronicoService->excluirNota($row);
            $this->atualizaPercurso($request, $row);
        }
    }
    
    public function savePercursoMdfe (Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        if($row["__is_new"]) {
            $this->manifestoDocumentoEletronicoService->setSequencialPercurso($row);
        }
        $this->dataSourceName = "percursomdfe";
        parent::save($request, $response);
    }
    
    public function mudarOrdemPercursoMdfe (Request\Row $request, Response $response) {
        $row = $request->getRow();
        $this->manifestoDocumentoEletronicoService->mudarOrdemPercurso($row);
    }
    
    public function excluirPercursoMdfe (Request\Row $request, Response $response) {
        $row = $request->getRow();
        $notaRelacionada = $this->verificaPercurso($request, $row);
        if ($notaRelacionada) {
            $response->setError(new Response\Error("Não é possível remover. Existem notas relacionadas com destino neste município.", '11'));
            return;
        }
        $this->manifestoDocumentoEletronicoService->excluirPercurso($row);
    }
    
    public function excluirPercursoMdfeMultiplo (Request\Dataset $request, Response $response) {
        $dataset = $request->getDataset();
        $rows = $dataset->getRows();
        foreach($rows as $row) {
            $this->manifestoDocumentoEletronicoService->excluirPercurso($row);
        }
    }
    
    public function atualizaPercurso($request, $row) {
        $mdfe = $this->montaMdfe($request, $row);
        $this->manifestoDocumentoEletronicoService->limparPercurso($mdfe);
        foreach ($mdfe["NOTAS_RELACIONADAS"] as $nota) {
           $status = $this->manifestoDocumentoEletronicoService->atualizaPercurso($nota);
           if (!$status) {
               $this->manifestoDocumentoEletronicoService->setSequencialPercurso($nota);
               $saveRequest = new Request\Row($nota, "POST", '', $request->getUserId());
               $response = new Response();
               $this->dataSourceName = "percursomdfe";
               parent::save($saveRequest, $response);
           }
        }
    }
    
    public function verificaPercurso($request, $row) {
        $mdfe = $this->montaMdfe($request, $row);
        foreach ($mdfe["NOTAS_RELACIONADAS"] as $nota) {
            if ($nota["CDMUNICIPIODESTINONF"] == $row["CDMUNICIPIO"] && $nota["SGESTADODESTINONF"] == $row["SGESTADO"]) {
                return true; //Existem notas destinadas para o percurso escolhido
            }
        }
        return false;
    }
    
    public function montaMdfe ($request, $cabecalho = null) {
        $responseTransmissao = new Response();
        $i = 0;
        
        if (!$cabecalho) {
            $this->dataSourceName = "mdfe_emissao";
            $conditions = $request->getFilterCriteria()->getConditions();
            parent::find($request, $responseTransmissao);
            
            $cabecalho = $responseTransmissao->getDatasets()[$i++]->getRows()[0];
        }
        
        $conditions = [];
        array_push($conditions, array (
            "columnName"     => "CDVIATURA",
            "operator" => "=",
            "value"    => $cabecalho["CDVIATURA"]));
            
        $filterCriteria = new FilterCriteria('', $conditions);
        $requestTransmissao = new Request\Filter($filterCriteria, "POST", '', $request->getUserId());
        $this->dataSourceName = "viatura";
        parent::find($requestTransmissao, $responseTransmissao);
            
        array_push($conditions, array (
            "columnName"     => "NRLANCTOMDFE",
            "operator" => "=",
            "value"    => $cabecalho["NRLANCTOMDFE"]));
            
        $filterCriteria = new FilterCriteria('', $conditions);
        $requestTransmissao = new Request\Filter($filterCriteria, "POST", '', $request->getUserId());

        $this->dataSourceName = "notasrelacionadasmdfe";
        parent::find($requestTransmissao, $responseTransmissao);
        $this->dataSourceName = "mdfe_percurso";
        parent::find($requestTransmissao, $responseTransmissao);
        $this->dataSourceName = "motoristamdfe_motorista";
        parent::find($requestTransmissao, $responseTransmissao);
        $this->dataSourceName = "reboquemdfe_reboque_viatura";
        parent::find($requestTransmissao, $responseTransmissao);
        $this->dataSourceName = "segurocargamdfe";
        parent::find($requestTransmissao, $responseTransmissao);
        $this->dataSourceName = "dadosadicmdfe";
        parent::find($requestTransmissao, $responseTransmissao);
        
        $viatura = $responseTransmissao->getDatasets()[$i++]->getRows()[0];
        $notasRelacionadas = $responseTransmissao->getDatasets()[$i++]->getRows();
        $percurso = $responseTransmissao->getDatasets()[$i++]->getRows();
        $motoristas = $responseTransmissao->getDatasets()[$i++]->getRows();
        $reboques = $responseTransmissao->getDatasets()[$i++]->getRows();
        $seguroCarga = $responseTransmissao->getDatasets()[$i++]->getRows();
        $dadosAdicionais = $responseTransmissao->getDatasets()[$i++]->getRows();
        
        foreach($seguroCarga as &$seguro) {
            $responseTemp = new Response();
            $conditionsTemp = $conditions;
            array_push($conditionsTemp, array (
                "columnName"     => "NRSEQSEGCARMDFE",
                "operator" => "=",
                "value"    => $seguro["NRSEQSEGCARMDFE"]));
                
            $filterTemp = new FilterCriteria('', $conditionsTemp);
            $requestTemp = new Request\Filter($filterTemp, "POST", '', $request->getUserId());
            $this->dataSourceName = "averbacaomdfe";
            parent::find($requestTemp, $responseTemp);
            
            $averbacao = $responseTemp->getDatasets()[0]->getRows();
            $seguro["AVERBACAO"] = $averbacao;
        }
        
        foreach($reboques as &$reboque) {
            $responseTemp = new Response();
            $conditionsTemp = array();
            array_push($conditionsTemp, array (
                "columnName"     => "CDVIATURA",
                "operator" => "=",
                "value"    => $reboque["CDREBOQUE"]));
            
            $filterTemp = new FilterCriteria('', $conditionsTemp);
            $requestTemp = new Request\Filter($filterTemp, "POST", '', $request->getUserId());
            $this->dataSourceName = "viatura";
            parent::find($requestTemp, $responseTemp);
            
            $viaturaReboque = $responseTemp->getDatasets()[0]->getRows()[0];
            $reboque["VIATURA"] = $viaturaReboque;
        }
        
        foreach ($percurso as &$rowPercurso) {
            $rowPercurso["DESCARGA"] = false;
        }
            
        foreach ($notasRelacionadas as $nota) {
            foreach ($percurso as &$rowPercurso) {
                if ($nota["CDMUNICIPIODESTINONF"] == $rowPercurso["CDMUNICIPIO"] && $nota["SGESTADODESTINONF"] == $rowPercurso["SGESTADO"]) {
                    $rowPercurso["DESCARGA"] = true;
                }
            }
        }
        
        $mdfe = $cabecalho;
        $mdfe["VIATURA"] = $viatura;
        $mdfe["NOTAS_RELACIONADAS"] = $notasRelacionadas;
        $mdfe["PERCURSO"] = $percurso;
        $mdfe["MOTORISTAS"] = $motoristas;
        $mdfe["REBOQUES"] = $reboques;
        $mdfe["SEGURO_CARGA"] = $seguroCarga;
        $mdfe["DADOS_ADICIONAIS"] = $dadosAdicionais;
        
        return $mdfe;
    }
    
    public function transmiteMdfe (Request\Filter $request, Response $response) {
        $erroMdfe;
        
        $mdfe = $this->montaMdfe($request);
        
        $this->manifestoDocumentoEletronicoService->atualizaDadosNotas($mdfe);
        
        try {
            if (!$mdfe["NRMDFE"]) {
                $this->manifestoDocumentoEletronicoService->setSequencialNrmdfe($mdfe);
            }
            
            $this->manifestoDocumentoEletronicoService->setChaveAcessoMdfe($mdfe); 
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error("Erro ao salvar o Sequencial/Chave de Acesso do MDF-e: " . $e->getMessage(), $e->getCode()));
            return;
        }
        
        try {
            $mdfeValido = $this->manifestoDocumentoEletronicoService->validaMdfe($mdfe, $erroMdfe);
            if (!$mdfeValido) {
                $response->setError(new Response\Error("MDF-e inválido: " . $erroMdfe, 10));
                return;
            } 
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error("Erro ao verificar a validade do MDF-e: " . $e->getMessage(), $e->getCode()));
            return;
        }
        
        try {
           $config = $this->getConfig($mdfe); 
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error("Erro ao obter a configuração para transmissão: " . $e->getMessage(), $e->getCode()));
            return;
        }
        
        try {
            $xml = $this->manifestoDocumentoEletronicoService->criaXmlTransmissaoMdfe($mdfe);
            $this->manifestoDocumentoEletronicoService->salvaXml($mdfe, self::TIPO_ARQ_XML_AUTORIZACAO, $xml);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error("Erro ao criar o XML: " . $e->getMessage(), $e->getCode()));
            return;
        }

        try {
            $respostaTransmissao = $this->manifestoDocumentoEletronicoService->transmiteMdfe($xml, $mdfe["CDEMPRESA"], $mdfe["NRLOTEMDFE"], $config);
            sleep(3);
            $respostaStatus = $this->manifestoDocumentoEletronicoService->consultaChave($mdfe["NRACESSOMDFE"], $mdfe["CDEMPRESA"], $config);
        }
        catch (NFePHP\Common\Certificate\Exception\Expired $e) {
            $response->setError(new Response\Error("Erro ao transmitir o XML: O Certificado está Vencido.", $e->getCode()));
            return;
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error("Erro ao transmitir o XML: " . $e->getMessage(), $e->getCode()));
            return;
        }
        
        $responseMessage = $this->getResponseMessage('TRANSMITIR', $respostaTransmissao, $respostaStatus, $mdfe["NRACESSOMDFE"], $mdfe["NRMDFE"]);
        if ($responseMessage["statusMdfe"] == 'R') {
            $responseMessage["isError"] = true;
        }
        $response->addMessage(new Response\Message($responseMessage));
        
        $this->manifestoDocumentoEletronicoService->setStatusMdfe($mdfe, $responseMessage["statusMdfe"]);
    }
    
    public function cancelaMdfe (Request\Filter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        foreach ($conditions as $index => $condition) {
            if ($condition["columnName"] == "MOTCANCELA") {
                $motivoCancelamento = $condition["value"];
                unset($conditions[$index]);
                break;
            }
        }
        
        $requestMdfe = new Request\Filter(new FilterCriteria('', $conditions), "POST", '', $request->getUserId());
        
        $mdfe = $this->montaMdfe($requestMdfe);
        $mdfe["MOTCANCELA"] = $motivoCancelamento;
        
        $config = $this->getConfig($mdfe);
        
        if(!$mdfe["NRACESSOMDFE"]) {
            $response->setError(new Response\Error("Erro ao realizar o cancelamento: O MDF-e não possui Chave de Acesso relacionada.", '10'));
            return;
        }
        
        try {
            $respostaCancelamento = $this->manifestoDocumentoEletronicoService->cancelaMdfe($mdfe["NRACESSOMDFE"], $mdfe["CDEMPRESA"], $config, $motivoCancelamento);
            sleep(3);
            $respostaStatus = $this->manifestoDocumentoEletronicoService->consultaChave($mdfe["NRACESSOMDFE"], $mdfe["CDEMPRESA"], $config);
        }
        catch (NFePHP\Common\Certificate\Exception\Expired $e) {
            $response->setError(new Response\Error("Erro ao cancelar: O Certificado está Vencido.", $e->getCode()));
            return;
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error("Erro ao cancelar: " . $e->getMessage(), $e->getCode()));
            return;
        }
            
        $responseMessage = $this->getResponseMessage('CANCELAR', $respostaCancelamento, $respostaStatus, $mdfe["NRACESSOMDFE"], $mdfe["NRMDFE"]);
        if (in_array($responseMessage["statusMdfe"], ['P', 'T', 'A'])) {
            $responseMessage["isError"] = true;
        }
        $response->addMessage(new Response\Message($responseMessage));
        
        $this->manifestoDocumentoEletronicoService->setStatusMdfe($mdfe, $responseMessage["statusMdfe"]);
    }
    
    public function encerraMdfe (Request\Filter $request, Response $response) {
        $mdfe = $this->montaMdfe($request);
        
        $config = $this->getConfig($mdfe);
        
        if(!$mdfe["NRACESSOMDFE"]) {
            $response->setError(new Response\Error("Erro ao realizar o encerramento: O MDF-e não possui Chave de Acesso relacionada.", '10'));
            return;
        }
        
        try {
            $respostaEncerramento = $this->manifestoDocumentoEletronicoService->encerraMdfe($mdfe["NRACESSOMDFE"], $mdfe["CDEMPRESA"], $mdfe["CDESTADOIBGE"], $mdfe["CDMUNICIBGE"], $config);
            sleep(3);
            $respostaStatus = $this->manifestoDocumentoEletronicoService->consultaChave($mdfe["NRACESSOMDFE"], $mdfe["CDEMPRESA"], $config);
        }
        catch (NFePHP\Common\Certificate\Exception\Expired $e) {
            $response->setError(new Response\Error("Erro ao encerrar: O Certificado está Vencido.", $e->getCode()));
            return;
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error("Erro ao encerrar: " . $e->getMessage(), $e->getCode()));
            return;
        }
        
        $responseMessage = $this->getResponseMessage('ENCERRAR', $respostaEncerramento, $respostaStatus, $mdfe["NRACESSOMDFE"], $mdfe["NRMDFE"]);
        if (in_array($responseMessage["statusMdfe"], ['P', 'T', 'A'])) {
            $responseMessage["isError"] = true;
        }
        $response->addMessage(new Response\Message($responseMessage));
        
        $this->manifestoDocumentoEletronicoService->setStatusMdfe($mdfe, $responseMessage["statusMdfe"]);
    }
    
    public function consultaMdfe (Request\Filter $request, Response $response) {
        $mdfe = $this->montaMdfe($request);
        
        $config = $this->getConfig($mdfe);
        
        if(!$mdfe["NRACESSOMDFE"]) {
            $response->setError(new Response\Error("Erro ao verificar o status: O MDF-e não possui Chave de Acesso relacionada.", '10'));
            return;
        }
        
        try {
            $respostaStatus = $this->manifestoDocumentoEletronicoService->consultaChave($mdfe["NRACESSOMDFE"], $mdfe["CDEMPRESA"], $config);
        }
        catch (NFePHP\Common\Certificate\Exception\Expired $e) {
            $response->setError(new Response\Error("Erro ao consultar: O Certificado está Vencido.", $e->getCode()));
            return;
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error("Erro ao consultar: " . $e->getMessage(), $e->getCode()));
            return;
        }
        
        $responseMessage = $this->getResponseMessage('CONSULTAR', $respostaStatus, $respostaStatus, $mdfe["NRACESSOMDFE"], $mdfe["NRMDFE"]);
        $response->addMessage(new Response\Message($responseMessage));
        
        $this->manifestoDocumentoEletronicoService->setStatusMdfe($mdfe, $responseMessage["statusMdfe"]);
    }
    
    public function getResponseMessage($acao, $responseAcao, $respostaStatus, $chaveAcesso, $nrmdfe) {
        $codigo = $respostaStatus->cStat;
        $motivo = $responseAcao->xMotivo;
        
        if ($codigo == '100') {
            $statusMdfe = 'A';
        }
        else if ($codigo == '101') {
            $statusMdfe = 'C';
        }
        else if (in_array($codigo, ['103', '104', '105'])) {
            $statusMdfe = 'T';
        }
        else if ($codigo == '132') {
            $statusMdfe = 'E';
        }
        /*
        else if (in_array($codigo, ['106', '217'])) {
        	$statusMdfe = 'P';
        }
        */
        else {
            $statusMdfe = 'R';
        }
        
        $message = "MDFE_ACAO_" . $acao . "_RESULTADO_" . $statusMdfe;
        
        $responseMessage = array(
            "message" => $message, 
            "statusMdfe" => $statusMdfe,
            "codigo" => $codigo,
            "motivo" => $motivo,
            "chaveAcesso" => $chaveAcesso,
            "nrmdfe" => $nrmdfe,
            "isError" => false);
        return $responseMessage;
    }
    
    public function getConfig($mdfe) {
        return [
            "atualizacao" => date('Y-m-d H:i:s'),
            "tpAmb" => $mdfe["IDTPAMBIENTEMDFE"],
            "razaosocial" => $mdfe["NMRAZSOCEMPR"],
            "cnpj" => $mdfe["NRINSJURIEST"],
            "ie" => $mdfe["CDINSCESTA"],
            "siglaUF" => $mdfe["SGESTADO"],
            "versao" => '3.00'
        ];
    }
    
    public function downloadDamdfe (Request\Filter $request, Response $response) {
        $mdfe = $this->montaMdfe($request);
        
        $config = $this->getConfig($mdfe);
        
        try {
            $xml = $this->manifestoDocumentoEletronicoService->criaXmlTransmissaoMdfe($mdfe);
            $downloadDamdfe = $this->manifestoDocumentoEletronicoService->downloadDamdfe($xml, $mdfe["NRACESSOMDFE"]);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('downloadDamdfe', array($downloadDamdfe)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}
