<?php

namespace Teknisa\DF\Services;

use Doctrine\ORM\EntityManager;
use Financeiro\Entities\Boletreceber;
use Teknisa\DF\Services\Emissao\GeraEmissaoBoletoService;
use Teknisa\DF\Services\Emissao\ValidacaoEmissaoBoletoService;

/**
 * Class EmissaoBoletoService
 * @package Teknisa\GER\Modules\Financeiro\Services
 */
class EmissaoBoletoService{

    protected $entityManager;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    /**
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @return null
     * @throws \Exception
     */
    public function emiteBoleto($nrorg, $cdfilial, $nrlanbolrec)
    {
        try {
            /** @var $boleto Boletreceber */
            $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg($nrlanbolrec, $cdfilial, $nrorg);
            // Emissão do Boleto a Receber
            $emissaoBoleto = GeraEmissaoBoletoService::factory($boleto);

            //VerificaÃ§Ã£o se o boleto ja foi emitido.
            if ($boleto->getIdstatbrec() != 'E'){

                // Executa validacoes quanto a emissao de boleto. Esta validacao pode variar de acordo com bancos distintos.
                $validacao = ValidacaoEmissaoBoletoService::factory($boleto);
                $validacao->valida();
                $nossoNumero    = $boleto->getCdnosnumrec()?:$emissaoBoleto->getNossoNumeroComDigito();

                $codigoDeBarras = $emissaoBoleto->getCodigoDeBarras($nossoNumero);
                $linhaDigitavel = $emissaoBoleto->getLinhaDigitavel($nossoNumero);

                $boleto->setCdnosnumrec(substr($nossoNumero,0,-1).' '.substr($nossoNumero,-1));
                $boleto->setDslinhabarrec($linhaDigitavel);
                $boleto->setCdbarrarec($codigoDeBarras);
                $boleto->setIdstatbrec('E');

                $boleto->persist();
                \Setforms2\Shared\Registry::getEntityManager()->flush();
            }

            return $emissaoBoleto->getRelatorioBoleto($boleto->getNrorg(),$boleto->getCdreducont());
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }
    
    public function cancelaEmissao($nrorg, $cdfilial, $nrlanbolrec) {
         try {
             
            $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg($nrlanbolrec, $cdfilial, $nrorg);
            $boleto->setCdnosnumrec('');
            $boleto->setDslinhabarrec('');
            $boleto->setCdbarrarec('');
            $boleto->setIdstatbrec('A'); 
            $boleto->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();
            
         } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }
}